@@ -26,18 +26,26 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 /*@1@*/
 
 #include <ctype.h>
+#include <string.h>
 
 // prj
 #include "globals.h"
 #include "utils.h"
 
 // sim.src
-#include "uccl.h"
+//#include "uccl.h"
 
 // local, cmd.src
 #include "cmd_uccl.h"
 
 
+static struct id_element cpu_states[]= {
+  { stGO,	"OK" },
+  { stIDLE,	"Idle" },
+  { stPD,	"PowerDown" },
+  { 0, 0 }
+};
+
 /*
  * Command: state
  *----------------------------------------------------------------------------
@@ -48,10 +56,10 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 //		      class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_state_cmd)
 {
-  con->dd_printf("CPU state= %s PC= 0x%06x XTAL= %g\n",
+  con->dd_printf("CPU state= %s PC= 0x%06x frequency= %.0f HZ\n",
 		 get_id_string(cpu_states, uc->state),
 		 AU(uc->PC), 
-		 uc->xtal);
+		 uc->get_xtal());
   con->dd_printf("Operation since last reset= (%lu vclks)\n",
 		 (unsigned long)(uc->vc.fetch) +
 		 (unsigned long)(uc->vc.rd) +
@@ -60,20 +68,22 @@ COMMAND_DO_WORK_UC(cl_state_cmd)
   con->dd_printf("Fetch= %lu ", (unsigned long)(uc->vc.fetch));
   con->dd_printf("Read= %lu ", (unsigned long)(uc->vc.rd));
   con->dd_printf("Write= %lu\n", (unsigned long)(uc->vc.wr));
-  con->dd_printf("Total time since last reset= %g sec (%lu clks)\n",
-		 uc->get_rtime(), (unsigned long)(uc->ticks->ticks));
-  con->dd_printf("Time in isr = %g sec (%lu clks) %3.2g%%\n",
-		 uc->isr_ticks->get_rtime(uc->xtal),
-		 uc->isr_ticks->ticks,
-		 (uc->ticks->ticks == 0)?0.0:
-		 (100.0*((double)(uc->isr_ticks->ticks)/
-			 (double)(uc->ticks->ticks))));
-  con->dd_printf("Time in idle= %g sec (%lu clks) %3.2g%%\n",
-		 uc->idle_ticks->get_rtime(uc->xtal),
-		 uc->idle_ticks->ticks,
-		 (uc->ticks->ticks == 0)?0.0:
-		 (100.0*((double)(uc->idle_ticks->ticks)/
-			 (double)(uc->ticks->ticks))));
+
+  con->dd_printf("Total time since last reset= %.15f sec (%lu clks)\n",
+		 uc->ticks->get_rtime(), (unsigned long)(uc->ticks->get_ticks()));
+
+  con->dd_printf("Time in isr = %.15f sec (%lu clks) %3.2f%%\n",
+		 uc->isr_ticks->get_rtime(),
+		 uc->isr_ticks->get_ticks(),
+		 (uc->ticks->get_ticks() == 0 ? 0.0 :
+		   (100.0 * uc->isr_ticks->get_rtime() / uc->ticks->get_rtime())));
+
+  con->dd_printf("Time in idle= %.15f sec (%lu clks) %3.2f%%\n",
+		 uc->idle_ticks->get_rtime(),
+		 uc->idle_ticks->get_ticks(),
+		 (uc->ticks->get_ticks() == 0 ? 0.0 :
+		   (100.0 * uc->idle_ticks->get_rtime() / uc->ticks->get_rtime())));
+
   con->dd_printf("Max value of stack pointer= 0x%06x, avg= 0x%06x\n",
 		 AU(uc->sp_max), AU(uc->sp_avg));
   con->dd_printf("Simulation: %s\n",
@@ -98,17 +108,19 @@ COMMAND_DO_WORK_UC(cl_file_cmd)
 {
   const char *fname= 0;
   long l;
-  
+
   if ((cmdline->param(0) == 0) ||
       ((fname= cmdline->param(0)->get_svalue()) == NULL))
     {
       con->dd_printf("File name is missing.\n");
       return(0);
     }
-  
+
   if ((l= uc->read_file(fname, con)) >= 0)
-    ;//con->dd_printf("%ld words read from %s\n", l, fname);
-    
+    {
+      //con->dd_printf("%ld words read from %s\n", l, fname);
+    }
+
   return(0);
 }
 
@@ -168,7 +180,7 @@ COMMAND_DO_WORK_UC(cl_pc_cmd)
 	}
       if (!uc->inst_at(addr))
 	con->dd_printf("Warning: maybe not instruction at 0x%06x\n", AU(addr));
-      uc->PC= addr;
+      uc->set_PC(addr);
     }
   uc->print_disass(uc->PC, con);
   return(false);
@@ -198,6 +210,32 @@ CMDHELP(cl_reset_cmd,
 	"Reset processor to start state",
 	"long help of reset")
 
+/*
+ * Command: tick
+ *----------------------------------------------------------------------------
+ */
+
+COMMAND_DO_WORK_UC(cl_tick_cmd)
+{
+  class cl_cmd_arg *params[1];
+  params[0]= cmdline->param(0);
+  int cycles= 1;
+
+  if (params[0] != NULL)
+    cycles= params[0]->i_value;
+
+  uc->tick(cycles);
+
+  return 0;
+}
+
+CMDHELP(cl_tick_cmd,
+	"tick [number]",
+	"Tick a given number of clock cycles without execution",
+	"Ticks a given number of clock cycles. Timers and HW update but no"
+	" execution takes place.")
+
+
 /*
  * Command: dump
  *----------------------------------------------------------------------------
@@ -208,35 +246,69 @@ CMDHELP(cl_reset_cmd,
 //		     class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_dump_cmd)
 {
-  class cl_memory *mem= 0;
-  long bpl= 8;
-  t_addr start= 0, end;
+  class cl_memory *mem= uc->rom;
+  t_addr start = -1, end = -1;
+  long bpl= -1;
+
   class cl_cmd_arg *params[4]= { cmdline->param(0),
 				 cmdline->param(1),
 				 cmdline->param(2),
 				 cmdline->param(3) };
-  /*enum dump_format*/int fmt= df_hex;
-  
+  char fmt = 0;
+
   if (params[0] &&
-      params[0]->as_bit(uc))
+      params[0]->as_string())
+    {
+      char *s= params[0]->get_svalue();
+      if (s && *s && s[0] == '/')
+        {
+          for (size_t i = 1; i < strlen(s); i++)
+            {
+              char c = tolower(s[i]);
+              switch (c)
+                {
+		case 'b':
+		  if (con->get_fout() && con->get_fout()->tty)
+		    {
+		      con->dd_printf("Error: binary format not supported on tty\n");
+		      return false;
+		    }
+		  break;
+		case 'x':
+		case 'h': // hex
+		case 'i': // ihex
+		case 's': // string
+		  break;
+		default:
+		  con->dd_printf("Error: unknown format option '%c'\n", c);
+		  return false;
+                }
+              fmt = c;
+            }
+          cmdline->params->free_at(0);
+          params[0]= cmdline->param(0);
+          params[1]= cmdline->param(1);
+          params[2]= cmdline->param(2);
+          params[3]= cmdline->param(3);
+        }
+    }
+
+  if (params[0] &&
+      params[0]->as_bit(uc) &&
+      params[0]->value.bit.bitnr_low >= 0)
     {
       int i= 0;
       while (params[0] &&
 	     params[0]->as_bit(uc))
 	{
-	  t_mem m;
-	  mem= params[0]->value.bit.mem;
-	  m= mem->read(params[0]->value.bit.mem_address);
-	  char *sn=
-	    uc->symbolic_bit_name((t_addr)-1,
-				  mem,
-				  params[0]->value.bit.mem_address,
-				  params[0]->value.bit.mask);
-	  con->dd_printf("%10s ", sn?sn:"");
-	  con->dd_printf(mem->addr_format, params[0]->value.bit.mem_address);
-	  con->dd_printf(" ");
-	  con->dd_printf(mem->data_format, m);
-	  con->dd_printf(" %c\n", (m&(params[0]->value.bit.mask))?'1':'0');
+          if (!fmt)
+	    params[0]->value.bit.mem->dump(params[0]->value.bit.mem_address,
+                                           params[0]->value.bit.bitnr_high,
+                                           params[0]->value.bit.bitnr_low,
+                                           con);
+          else
+            con->dd_printf("Format options may not be specified for bits\n");
+
 	  i++;
 	  params[0]= cmdline->param(i);
 	}
@@ -244,84 +316,73 @@ COMMAND_DO_WORK_UC(cl_dump_cmd)
 	syntax_error(con);
       return false;
     }
-  if (params[0] &&
-      params[0]->as_string())
-    {
-      char *s= params[0]->get_svalue();
-      if (s && *s &&
-	  (strlen(s) > 1) &&
-	  (s[0]=='/'))
-	{
-	  size_t i;
-	  for (i= 0; i < strlen(s); i++)
-	    s[i]= tolower(s[i]);
-	  switch (tolower(s[1]))
-	    {
-	    case 's': fmt= df_string; break;
-	    case 'h': fmt= df_hex; break;
-	    case 'i': fmt= df_ihex; bpl= 32; break;
-	    case 'b':
-	      if (con->get_fout() &&
-		  con->get_fout()->tty)
-		return con->dd_printf("Error: binary format not supported on tty\n"),
-		  false;
-	      fmt= df_binary;
-	      break;
-	    }
-	  if (strlen(s) > 2)
-	    for (i= 2; i < strlen(s); i++)
-	      {
-		switch (s[i])
-		  {
-		  case 'l': fmt|= df_little; break;
-		  case 'b': fmt|= df_big; break;
-		  case '1': fmt|= df_1; break;
-		  case '2': fmt|= df_2; break;
-		  case '4': fmt|= df_4; break;
-		  case '8': fmt|= df_8; break;
-		  }
-	      }
-	  cmdline->shift();
-	  params[0]= cmdline->param(0);
-	  params[1]= cmdline->param(1);
-	  params[2]= cmdline->param(2);
-	  params[3]= cmdline->param(3);
-	}
+
+  if (params[0] == 0)
+    ;
+  else if (cmdline->syntax_match(uc, BIT)) {
+    mem= params[0]->value.bit.mem;
+    start= params[0]->value.bit.mem_address;
+  }
+  else if (cmdline->syntax_match(uc, BIT BIT)) {
+    mem= params[0]->value.bit.mem;
+    if (mem != params[1]->value.bit.mem) {
+      con->dd_printf("Start and end must be in the same address space\n");
+      return false;
     }
-  
-  enum dump_format df= (enum dump_format)fmt;
-  if ((cmdline->param(0)==NULL) ||
-      (!(cmdline->param(0)->as_memory(uc))))
-    {
-      con->dd_printf("No memory specified. Use \"info memory\" for available memories\n");
-      return(false);
-    }
-  if (cmdline->syntax_match(uc, MEMORY))
-    {
-      mem= cmdline->param(0)->value.memory.memory;
-      mem->dump(df, -1, -1, bpl, con/*->get_fout()*/);
+    start= params[0]->value.bit.mem_address;
+    end= params[1]->value.bit.mem_address;
+  }
+  else if (cmdline->syntax_match(uc, BIT BIT NUMBER)) {
+    mem= params[0]->value.bit.mem;
+    if (mem != params[1]->value.bit.mem) {
+      con->dd_printf("Start and end must be in the same address space\n");
+      return false;
     }
+    start= params[0]->value.bit.mem_address;
+    end= params[1]->value.bit.mem_address;
+    bpl  = params[2]->value.number;
+  }
+  else if (cmdline->syntax_match(uc, MEMORY)) {
+      mem= params[0]->value.memory.memory;
+  }
   else if (cmdline->syntax_match(uc, MEMORY ADDRESS)) {
-    mem  = cmdline->param(0)->value.memory.memory;
-    start= cmdline->param(1)->value.address;
-    end  = start+10*8-1;
-    mem->dump(df, start, end, bpl, con/*->get_fout()*/);
+    mem  = params[0]->value.memory.memory;
+    start= params[1]->value.address;
   }
   else if (cmdline->syntax_match(uc, MEMORY ADDRESS ADDRESS)) {
-    mem  = cmdline->param(0)->value.memory.memory;
-    start= cmdline->param(1)->value.address;
-    end  = cmdline->param(2)->value.address;
-    mem->dump(df, start, end, bpl, con/*->get_fout()*/);
+    mem  = params[0]->value.memory.memory;
+    start= params[1]->value.address;
+    end  = params[2]->value.address;
   }
   else if (cmdline->syntax_match(uc, MEMORY ADDRESS ADDRESS NUMBER)) {
-    mem  = cmdline->param(0)->value.memory.memory;
-    start= cmdline->param(1)->value.address;
-    end  = cmdline->param(2)->value.address;
-    bpl  = cmdline->param(3)->value.number;
-    mem->dump(df, start, end, bpl, con/*->get_fout()*/);
+    mem  = params[0]->value.memory.memory;
+    start= params[1]->value.address;
+    end  = params[2]->value.address;
+    bpl  = params[3]->value.number;
   }
-  else
+  else {
     syntax_error(con);
+    return false;
+  }
+
+  switch (fmt)
+    {
+    case 0: // default
+      mem->dump(1, start, end, bpl, con);
+      break;
+    case 'b': // binary
+      mem->dump_b(start, end, bpl, con);
+      break;
+    case 'h': case 'x':// hex
+      mem->dump(0, start, end, bpl, con);
+      break;
+    case 'i': // ihex
+      mem->dump_i(start, end, 32, con);
+      break;
+    case 's': // string
+      mem->dump_s(start, end, bpl, con);
+      break;
+    }
 
   return(false);;
 }
@@ -341,7 +402,8 @@ CMDHELP(cl_dump_cmd,
 //		   class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_di_cmd)
 {
-  cmdline->insert_param(0, new cl_cmd_sym_arg("iram"));
+  cmdline->insert_param(0, new cl_cmd_sym_arg("/h"));
+  cmdline->insert_param(1, new cl_cmd_sym_arg("iram"));
   cl_dump_cmd::do_work(uc, cmdline, con);
   return(0);
 }
@@ -361,7 +423,8 @@ CMDHELP(cl_di_cmd,
 //		   class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_dx_cmd)
 {
-  cmdline->insert_param(0, new cl_cmd_sym_arg("xram"));
+  cmdline->insert_param(0, new cl_cmd_sym_arg("/h"));
+  cmdline->insert_param(1, new cl_cmd_sym_arg("xram"));
   cl_dump_cmd::do_work(uc, cmdline, con);
   return(0);
 }
@@ -381,7 +444,8 @@ CMDHELP(cl_dx_cmd,
 //		    class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_dch_cmd)
 {
-  cmdline->insert_param(0, new cl_cmd_sym_arg(/*"rom"*/uc->rom->get_name("rom")));
+  cmdline->insert_param(0, new cl_cmd_sym_arg("/h"));
+  cmdline->insert_param(1, new cl_cmd_sym_arg(/*"rom"*/uc->rom->get_name("rom")));
   cl_dump_cmd::do_work(uc, cmdline, con);
   return(0);
 }
@@ -401,7 +465,8 @@ CMDHELP(cl_dch_cmd,
 //		   class cl_cmdline *cmdline, class cl_console *con)
 COMMAND_DO_WORK_UC(cl_ds_cmd)
 {
-  cmdline->insert_param(0, new cl_cmd_sym_arg("sfr"));
+  cmdline->insert_param(0, new cl_cmd_sym_arg("/h"));
+  cmdline->insert_param(1, new cl_cmd_sym_arg("sfr"));
   cl_dump_cmd::do_work(uc, cmdline, con);
   return(0);
 }
@@ -476,6 +541,7 @@ COMMAND_DO_WORK_UC(cl_disassemble_cmd)
 {
   t_addr start, realstart;
   int offset= -1, dir, lines= 20;
+  bool run_analyze= false;
   class cl_cmd_arg *params[4]= { cmdline->param(0),
 				 cmdline->param(1),
 				 cmdline->param(2),
@@ -483,22 +549,30 @@ COMMAND_DO_WORK_UC(cl_disassemble_cmd)
 
   start= disass_last_stop;
   if (params[0] == 0) ;
-  else if (cmdline->syntax_match(uc, ADDRESS)) {
-    start= params[0]->value.address;
-  }
-  else if (cmdline->syntax_match(uc, ADDRESS NUMBER)) {
-    start= params[0]->value.address;
-    offset= params[1]->value.number;
-  }
-  else if (cmdline->syntax_match(uc, ADDRESS NUMBER NUMBER)) {
-    start= params[0]->value.address;
-    offset= params[1]->value.number;
-    lines= params[2]->value.number;
-  }
   else
     {
-      syntax_error(con);
-      return(false);
+      char *s= (char*)(cmdline->tokens->at(0));
+      if (s && *s && (s[0]=='+'))
+	{
+	  run_analyze= true;
+	}
+      if (cmdline->syntax_match(uc, ADDRESS)) {
+	start= params[0]->value.address;
+      }
+      else if (cmdline->syntax_match(uc, ADDRESS NUMBER)) {
+	start= params[0]->value.address;
+	offset= params[1]->value.number;
+      }
+      else if (cmdline->syntax_match(uc, ADDRESS NUMBER NUMBER)) {
+	start= params[0]->value.address;
+	offset= params[1]->value.number;
+	lines= params[2]->value.number;
+      }
+      else
+	{
+	  syntax_error(con);
+	  return(false);
+	}
     }
 
   if (lines < 1)
@@ -506,6 +580,8 @@ COMMAND_DO_WORK_UC(cl_disassemble_cmd)
       con->dd_printf("Error: wrong `lines' parameter\n");
       return(false);
     }
+  if (run_analyze)
+    uc->analyze(start);
   if (!uc->there_is_inst())
     return(false);
   realstart= start;
@@ -620,7 +696,7 @@ cl_where_cmd::do_real_work(class cl_uc *uc,
     while (found)
       {
 	if (con->get_fout())
-	  mem->dump(addr, addr+len-1, 8, con/*->get_fout()*/);
+	  mem->dump(0, addr, addr+len-1, -1, con);
 	addr++;
 	found= mem->search_next(case_sensitive, array, len, &addr);
       }
@@ -763,26 +839,41 @@ CMDHELP(cl_hole_cmd,
 
 COMMAND_DO_WORK_UC(cl_var_cmd)
 {
-  class cl_cmd_arg *params[4]= { cmdline->param(0),
+  class cl_cmd_arg *params[5]= { cmdline->param(0),
 				 cmdline->param(1),
 				 cmdline->param(2),
-				 cmdline->param(3) };
+				 cmdline->param(3),
+				 cmdline->param(4) };
   class cl_memory *m= NULL;
   t_addr addr= -1;
-  int bit= -1;
-  class cl_var *v;
-  
-  if (cmdline->syntax_match(uc, STRING MEMORY ADDRESS NUMBER))
+  int bitnr_low= -1;
+  int bitnr_high= -1;
+
+  if (cmdline->syntax_match(uc, STRING MEMORY ADDRESS NUMBER NUMBER))
     {
       m= params[1]->value.memory.memory;
       addr= params[2]->value.address;
-      bit= params[3]->value.number;
+      bitnr_low= bitnr_high= params[3]->value.number;
+      bitnr_high= params[4]->value.number;
+    }
+  else if (cmdline->syntax_match(uc, STRING MEMORY ADDRESS NUMBER))
+    {
+      m= params[1]->value.memory.memory;
+      addr= params[2]->value.address;
+      bitnr_low= bitnr_high= params[3]->value.number;
     }
   else if (cmdline->syntax_match(uc, STRING MEMORY ADDRESS))
     {
       m= params[1]->value.memory.memory;
       addr= params[2]->value.address;
     }
+  else if (cmdline->syntax_match(uc, STRING BIT))
+    {
+      m= params[1]->value.bit.mem;
+      addr= params[1]->value.bit.mem_address;
+      bitnr_low= params[1]->value.bit.bitnr_low;
+      bitnr_high= params[1]->value.bit.bitnr_high;
+    }
   else if (cmdline->syntax_match(uc, STRING CELL))
     {
       m= uc->address_space(params[1]->value.cell, &addr);
@@ -796,10 +887,6 @@ COMMAND_DO_WORK_UC(cl_var_cmd)
   if (!valid_sym_name(params[0]->value.string.string))
     return con->dd_printf("name is invalid\n"),
       false;
-  if ((bit >= 0) &&
-      (bit >= (int)sizeof(t_mem)*8))
-    return con->dd_printf("max bit number is %d\n", (int)sizeof(t_mem)*8),
-      false;
   
   if (m)
     if (!m->is_address_space())
@@ -809,41 +896,32 @@ COMMAND_DO_WORK_UC(cl_var_cmd)
     if (!m->valid_address(addr))
       return con->dd_printf("invalid address\n"),
 	false;
-  if (bit >= 0)
-    if (bit >= 32)
-      return con->dd_printf("invalid bit number\n"),
-	false;
-
-  if (uc->symbol2address(params[0]->value.string.string,
-			 (class cl_address_space **)NULL,
-			 (t_addr*)NULL))
-    return con->dd_printf("already exists\n"),
+  if (bitnr_low >= (int)sizeof(t_mem)*8 ||
+      bitnr_high >= (int)sizeof(t_mem)*8)
+    return con->dd_printf("max bit number is %d\n", (int)sizeof(t_mem)*8),
       false;
-  
+
   if (m)
-    {
-      v= new cl_var(params[0]->value.string.string,
-		    (cl_address_space*)m, addr, chars(""), bit);
-      v->init();
-      uc->vars->add(v);
-    }
+    uc->vars->add(params[0]->value.string.string, m, addr, bitnr_high, bitnr_low, "");
   else
     {
-      if (bit < 0)
+      if (bitnr_low < 0)
 	{
 	  if (addr < 0)
 	    {
-	      if (!uc->variables->search_cell(CELL_VAR, false, &addr))
+	      t_index i;
+	      for (addr= 0; addr < uc->variables->get_size(); addr++)
+		if (!uc->vars->by_addr.search(uc->variables, addr, i))
+		  break;
+	      if (addr == uc->variables->get_size())
 		return con->dd_printf("no space\n"),
 		  false;
 	    }
 	  if (!uc->variables->valid_address(addr))
 	    return con->dd_printf("out of range\n"),
 	      false;
-	  v= new cl_var(params[0]->value.string.string,
-			uc->variables, addr, chars(""), bit);
-	  v->init();
-	  uc->vars->add(v);
+          uc->vars->add(params[0]->value.string.string,
+                        uc->variables, addr, bitnr_high, bitnr_low, "");
 	}
       else
 	{
@@ -858,4 +936,63 @@ CMDHELP(cl_var_cmd,
 	"Create new variable",
 	"long help of var")
 
+/*
+ * Command: rmvar
+ *----------------------------------------------------------------------------
+ */
+
+COMMAND_DO_WORK_UC(cl_rmvar_cmd)
+{
+  if (cmdline->syntax_match(uc, STRING))
+    uc->vars->del(cmdline->param(0)->value.string.string);
+  else
+    return syntax_error(con), false;
+
+  return false;
+}
+
+CMDHELP(cl_rmvar_cmd,
+	"rmvar name",
+	"Remove variable",
+	"Deletes the name variable")
+
+/*
+ * Command: analyze
+ *----------------------------------------------------------------------------
+ */
+
+COMMAND_DO_WORK_UC(cl_analyze_cmd)
+{
+  if (cmdline->nuof_params() == 0)
+    uc->analyze_init();
+  else
+    for (int i = 0; i < cmdline->nuof_params(); i++)
+      {
+        class cl_cmd_arg *param = cmdline->param(i);
+        if (param)
+          {
+            if (param->as_bit(uc))
+              {
+                if (param->value.bit.mem == uc->rom)
+                  uc->analyze(param->value.bit.mem_address);
+                else
+                  {
+                    con->dd_printf("%s[", param->value.bit.mem->get_name());
+                    con->dd_printf(param->value.bit.mem->addr_format, param->value.bit.mem_address);
+                    con->dd_printf("]: addresses to analyze must be in %s\n", uc->rom->get_name());
+                  }
+              }
+            else
+              con->dd_printf("%s cannot be interpreted as a rom address\n", cmdline->tokens->at(i));
+          }
+      }
+
+  return false;
+}
+
+CMDHELP(cl_analyze_cmd,
+	"analyze [addr...]",
+	"Analyze reachable code globally or from the address(es) given",
+	"long help of analyze")
+
 /* End of cmd.src/cmd_uc.cc */
