@@ -28,17 +28,14 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #ifndef CMD_CMDUTIL_HEADER
 #define CMD_CMDUTIL_HEADER
 
+#include "ddconfig.h"
+
 #ifdef SOCKET_AVAIL
 # include HEADER_SOCKET
 #endif
 
-#include "ddconfig.h"
-
 
 extern char *proc_escape(char *string, int *len);
-extern int cmd_vfprintf(FILE *f, char *format, va_list ap);
-extern int cmd_fprintf(FILE *f, char *format, ...);
-
 extern int bool_name(char *s, int *val);
 
   
