@@ -25,20 +25,24 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA. */
 /*@1@*/
 
-#include "ddconfig.h"
+//#include "ddconfig.h"
 
+#include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
-#include "i_string.h"
+//#include <stdarg.h>
+#include <string.h>
+#include <unistd.h>
+
+//#include "i_string.h"
 
 // prj
-#include "fiocl.h"
-#include "utils.h"
+//#include "fiocl.h"
+//#include "utils.h"
 #include "appcl.h"
 
 // local, cmd
 #include "commandcl.h"
-#include "argcl.h"
+//#include "argcl.h"
 
 
 /*
@@ -47,7 +51,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  */
 
 cl_cmdline::cl_cmdline(class cl_app *the_app,
-		       char *acmd, class cl_console_base *acon):
+		       const char *acmd, class cl_console_base *acon):
   cl_base()
 {
   app= the_app;
@@ -157,10 +161,17 @@ cl_cmdline::split(void)
           strncpy(param_str, start, i);
 	  param_str[i]= '\0';
 	  tokens->add(strdup(param_str));
-	  if ((dot= strchr(param_str, '.')) != NULL)
+	  if ((dot= strchr(param_str, '[')) != NULL)
+            {
+              char *p;
+              for (p= dot+1; *p && *p != ']' && *p != ':'; p++);
+              if (*p == ':')
+	        split_out_bit(dot, param_str);
+              else
+                split_out_array(dot, param_str);
+            }
+	  else if ((dot= strchr(param_str, '.')) != NULL && dot != param_str)
 	    split_out_bit(dot, param_str);
-	  else if ((dot= strchr(param_str, '[')) != NULL)
-	    split_out_array(dot, param_str);
 	  else if (param_str[0] == '0' && param_str[1] == 'b')
             {
               long n= 0;
@@ -231,39 +242,89 @@ cl_cmdline::split_out_string(char **_start, char **_end)
 void
 cl_cmdline::split_out_output_redirection(char **_start, char **_end)
 {
-  char *start= *_start, *end/*= *_end*/;
-  int i;
+  char *start= *_start;
+  char *end;
+  int j;
   char mode[2];
 
   mode[0]= 'w';
   mode[1]= '\0';
   start++;
-  i= strcspn(start, " \t\v\r,");
-  end= start+i;
-  char *param_str= (char *)malloc(i+1);
-  char *n= param_str;
-  strncpy(param_str, start, i);
-  param_str[i]= '\0';
-  if (param_str &&
-      param_str[0] == '>')
+  if (*start=='>')
     {
-      n++;
       mode[0]= 'a';
+      start++;
     }
-  tokens->add(strdup(n));
-  con->redirect(n, mode);
-  free(param_str);
+  j= token_length(start);
+  end= start+j;
+  con->redirect(get_token(start).c_str(), mode);
   *_start= start;
   *_end= end;
 }
 
+void
+cl_cmdline::add_bit(char *dot, char *colon, class cl_cmd_arg *sfr)
+{
+  class cl_cmd_arg *bit_low = NULL, *bit_high = NULL;
+
+  if (colon)
+    {
+      *(colon++) = '\0';
+
+      char *end = strchr(colon, ']');
+
+      if (end)
+          *end = '\0';
+      else
+        {
+          con->dd_printf("Incomplete bit address\n");
+          delete sfr;
+          return;
+        }
+    }
+
+  if (*dot == '\0')
+    {
+      con->dd_printf("Incomplete bit address\n");
+      delete sfr;
+      return;
+    }
+
+  while (dot)
+    {
+      bit_high= bit_low;
+
+      if (strchr("0123456789", *dot) != NULL)
+       {
+         bit_low= new cl_cmd_int_arg((long)strtol(dot, 0, 0));
+         bit_low->init();
+       }
+      else
+       {
+         bit_low= new cl_cmd_sym_arg(dot);
+         bit_low->init();
+       }
+
+      dot= colon;
+      colon= NULL;
+    }
+
+  class cl_cmd_arg *arg;
+  params->add(arg= new cl_cmd_bit_arg(sfr, bit_low, bit_high));
+  arg->init();
+}
+
 void
 cl_cmdline::split_out_bit(char *dot, char *param_str)
 {
-  class cl_cmd_arg *sfr, *bit;
+  char *colon = NULL;
+  class cl_cmd_arg *sfr;
+
+  if (*dot == '[')
+    colon = strchr(dot+1, ':');
+
+  *(dot++) = '\0';
 
-  *dot= '\0';
-  dot++;
   if (strchr("0123456789", *param_str) != NULL)
     {
       sfr= new cl_cmd_int_arg((long)strtol(param_str, 0, 0));
@@ -274,28 +335,8 @@ cl_cmdline::split_out_bit(char *dot, char *param_str)
       sfr= new cl_cmd_sym_arg(param_str);
       sfr->init();
     }
-  if (*dot == '\0')
-    {
-      bit= 0;
-      con->dd_printf("Uncomplete bit address\n");
-      delete sfr;
-    }
-  else
-    {
-      if (strchr("0123456789", *dot) != NULL)
-       {
-         bit= new cl_cmd_int_arg((long)strtol(dot, 0, 0));
-         bit->init();
-       }
-      else
-       {
-         bit= new cl_cmd_sym_arg(dot);
-         bit->init();
-       }
-      class cl_cmd_arg *arg;
-      params->add(arg= new cl_cmd_bit_arg(sfr, bit));
-      arg->init();
-    }
+
+  add_bit(dot, colon, sfr);
 }
 
 void
@@ -318,44 +359,121 @@ cl_cmdline::split_out_array(char *dot, char *param_str)
   if (*dot == '\0')
     {
       aname= 0;
-      con->dd_printf("Uncomplete array\n");
+      con->dd_printf("Incomplete array\n");
     }
   else
     {
-      char *p;
-      p= dot + strlen(dot) - 1;
-      while (p > dot &&
-            *p != ']')
-       {
-         *p= '\0';
-         p--;
-       }
-      if (*p == ']')
-       *p= '\0';
-      if (strlen(dot) == 0)
-       {
-         con->dd_printf("Uncomplete array index\n");
-         delete aname;
-       }
-      else    
-       {
-         if (strchr("0123456789", *dot) != NULL)
-           {
-             aindex= new cl_cmd_int_arg((long)strtol(dot, 0, 0));
-             aindex->init();
-           }
-         else
-           {
-             aindex= new cl_cmd_sym_arg(dot);
-             aindex->init();
-           }
-         class cl_cmd_arg *arg;
-         params->add(arg= new cl_cmd_array_arg(aname, aindex));
-         arg->init();
-       }
+      char *p = strchr(dot, ']');
+      if (!p)
+        {
+          con->dd_printf("Missing ']' in array index\n");
+          delete aname;
+        }
+      else
+        {
+          *(p++)= '\0';
+          if (strlen(dot) == 0)
+            {
+              con->dd_printf("Incomplete array index\n");
+              delete aname;
+            }
+          else
+            {
+              if (strchr("0123456789", *dot) != NULL)
+                {
+                  aindex= new cl_cmd_int_arg((long)strtol(dot, 0, 0));
+                  aindex->init();
+                }
+              else
+                {
+                  aindex= new cl_cmd_sym_arg(dot);
+                  aindex->init();
+                }
+              class cl_cmd_arg *arg= new cl_cmd_array_arg(aname, aindex);
+              arg->init();
+              if (*p == '.' || *p == '[')
+                  add_bit(p+1, strchr(p+1, ':'), arg);
+              else
+                  params->add(arg);
+            }
+        }
     }
 }
 
+int
+cl_cmdline::token_length(char *start)
+{
+  if (start==NULL ||
+      *start=='\0')
+    return 0;
+  if (*start == '\"')
+    {
+      int i;
+      char c;
+      for (i=1; start[i]!=0 && start[i]!='\"'; i++)
+	{
+	  c= start[i];
+	  if (c == '\\')
+	    i+= 2;
+	}
+      return i+1;
+    }
+  else
+    return strcspn(start, " \t\v\r,;");
+}
+
+chars
+cl_cmdline::get_token(char *start)
+{
+  int i;
+  char c;
+  chars cs= chars();
+  if (start==NULL ||
+      *start=='\0')
+    return cs;
+  if (*start == '\"')
+    {
+      for (i=1; start[i]!=0 && start[i]!='\"'; i++)
+	{
+	  c= start[i];
+	  if (c == '\\')
+	    {
+	      i++;
+	      if (start[i])
+		{
+		  switch (start[i])
+		    {
+		    case 'a': cs+= '\a'; break;
+		    case 'b': cs+= '\b'; break;
+		    case 'e': cs+= '\x1b'; break;
+		    case 'f': cs+= '\f'; break;
+		    case 'n': cs+= '\n'; break;
+		    case 'r': cs+= '\r'; break;
+		    case 't': cs+= '\t'; break;
+		    case 'v': cs+= '\v'; break;
+		    case '\'': cs+= '\''; break;
+		    case '\"': cs+= '\"'; break;
+		    default:
+		      cs+= start[i];
+		      break;
+		    }
+		}
+	      //i++;
+	    }
+	  else
+	    cs+= c;
+	}
+      return cs;
+    }
+  else
+    {
+      int l= strcspn(start, " \t\v\r,;");
+      for (i=0; i<l; i++)
+	cs+= start[i];
+    }
+  return cs;
+}
+
 int
 cl_cmdline::shift(void)
 {
@@ -501,7 +619,7 @@ cl_cmdline::set_data_list(class cl_cmd_arg *parm, int *iparm)
 {
   class cl_cmd_arg *next_parm;
   int len, i, j;
-  t_mem *array;
+  t_mem *array, *temp;
 
   len= 0;
   array= 0;
@@ -519,7 +637,12 @@ cl_cmdline::set_data_list(class cl_cmd_arg *parm, int *iparm)
 	  if (!array)
 	    array= (t_mem*)malloc(sizeof(t_mem)*l);
 	  else
-	    array= (t_mem*)realloc(array, sizeof(t_mem)*(l+len));
+	    {
+	      temp= (t_mem*)realloc(array, sizeof(t_mem)*(l+len));
+	      if (temp == NULL)
+		break;
+	      array= temp;
+	    }
 	  for (j= 0; j < l; j++)
 	    {
 	      array[len]= s[j];
@@ -539,7 +662,12 @@ cl_cmdline::set_data_list(class cl_cmd_arg *parm, int *iparm)
 	  if (!array)
 	    array= (t_mem*)malloc(sizeof(t_mem));
 	  else
-	    array= (t_mem*)realloc(array, sizeof(t_mem)*(1+len));
+	    {
+	      temp= (t_mem*)realloc(array, sizeof(t_mem)*(1+len));
+	      if (temp == NULL)
+		break;
+	      array= temp;
+	    }
 	  array[len]= next_parm->value.data;
 	  len++;
 	}
@@ -630,7 +758,7 @@ cl_cmd::name_match(const char *aname, int strict)
     {
       for (i= 0; i < names->count; i++)
 	{
-	  char *n= (char*)(names->at(i));
+	  const char *n= names->at(i);
 	  if (strcmp(aname, n) == 0)
 	    return(1);
 	}
@@ -639,7 +767,7 @@ cl_cmd::name_match(const char *aname, int strict)
     {
       for (i= 0; i < names->count; i++)
 	{
-	  char *n= (char*)(names->at(i));
+	  const char *n= names->at(i);
 	  if (strstr(n, aname) == n)
 	    return(1);
 	}
@@ -701,7 +829,7 @@ int
 cl_cmd::do_work(class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing.\n",
-		 (char*)(names->at(0)));
+		 names->at(0));
   return(0);
 }
 
@@ -710,7 +838,7 @@ cl_cmd::do_work(class cl_app *app,
 		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on application.\n",
-		 (char*)(names->at(0)));
+		 names->at(0));
   return(0);
 }
 
@@ -719,7 +847,7 @@ cl_cmd::do_work(class cl_sim *sim,
 		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on simulator.\n",
-		 (char*)(names->at(0)));
+		 names->at(0));
   return(0);
 }
 
@@ -728,7 +856,7 @@ cl_cmd::do_work(class cl_uc *uc,
 		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on microcontroller.\n",
-		 (char*)(names->at(0)));
+		 names->at(0));
   return(0);
 }
 
@@ -741,7 +869,7 @@ cl_cmd::print_short(class cl_console_base *con)
     return;
   
   if (usage_help.nempty())
-    con->dd_printf("%s", (char*)usage_help);
+    con->dd_printf("%s", usage_help.c_str());
   if (l > 19)
     {
       con->dd_printf("\n");
@@ -754,10 +882,10 @@ cl_cmd::print_short(class cl_console_base *con)
     }
   if (short_help.nempty())
     {
-	con->dd_printf("%s", (char*)short_help);
+	con->dd_printf("%s", short_help.c_str());
     }
   else
-    con->dd_printf("%s", (char*)(names->at(0)));
+    con->dd_printf("%s", names->at(0));
   con->dd_printf("\n");
 }
   
@@ -903,11 +1031,11 @@ cl_super_cmd::work(class cl_app *app,
 	return(cmd->work(app, cmdline, con));
       int i;
       con->dd_printf("\"%s\" must be followed by the name of a subcommand\n"
-		     "List of subcommands:\n", (char*)(names->at(0)));
+		     "List of subcommands:\n", names->at(0));
       for (i= 0; i < commands->count; i++)
 	{
 	  cmd= (class cl_cmd *)(commands->at(i));
-	  //con->dd_printf("%s\n", (char*)cmd->short_help);
+	  //con->dd_printf("%s\n", cmd->short_help.c_str());
 	  cmd->print_short(con);
 	}
       return(0);
@@ -915,7 +1043,7 @@ cl_super_cmd::work(class cl_app *app,
   if ((cmd= commands->get_cmd(cmdline, con->is_interactive())) == NULL)
     {
       con->dd_printf("Undefined subcommand: \"%s\". Try \"help %s\".\n",
-		     cmdline->get_name(), (char*)(names->at(0)));
+		     cmdline->get_name(), names->at(0));
       return(0);
     }
   return(cmd->work(app, cmdline, con));
