@@ -5,6 +5,6 @@ extern unsigned long bl_s105_termination;
 extern unsigned long bl_s105_start;
 extern unsigned long bl_s105_finish;
 extern unsigned long bl_s105_length;
-extern unsigned char bl_s105[];
+extern u8_t bl_s105[];
 
 #endif /* BL_S105_H */
