@@ -1029,7 +1029,7 @@ algebraicOpts (iCode * ic, eBBlock * ebp)
       /* if subtracting the same thing then zero     */
       if (IC_LEFT (ic)->key == IC_RIGHT (ic)->key &&
         !IS_OP_VOLATILE (IC_LEFT (ic)))
-        {printf("Sub. at %d\n", ic->key);
+        {
           ic->op = '=';
           IC_RIGHT (ic) = operandFromLit (0);
           IC_LEFT (ic) = NULL;
@@ -1691,44 +1691,50 @@ ifxOptimize (iCode * ic, set * cseSet,
           ReplaceOpWithCheaperOp(&IC_COND (ic), pdop);
           (*change)++;
         }
-      else if(ic->prev &&  /* Remove unnecessary casts */
-        (ic->prev->op == '=' || ic->prev->op == CAST || ic->prev->op == '!') && IS_ITEMP (IC_RESULT (ic->prev)) &&
-        IC_RESULT (ic->prev)->key == IC_COND (ic)->key && bitVectnBitsOn (OP_USES (IC_RESULT (ic->prev))) <= 1)
+      else if (ic->prev &&  /* Remove unnecessary casts */
+               (ic->prev->op == '=' || ic->prev->op == CAST || ic->prev->op == '!') &&
+               IS_ITEMP (IC_RESULT (ic->prev)) &&
+               IC_RESULT (ic->prev)->key == IC_COND (ic)->key &&
+               bitVectnBitsOn (OP_USES (IC_RESULT (ic->prev))) <= 1)
         {
-          sym_link *type = operandType (IC_RESULT (ic->prev));
-          if (ic->prev->op != CAST || IS_BOOL (type) || bitsForType (operandType (IC_RIGHT (ic->prev))) < bitsForType (type))
-          {
-            if (!isOperandVolatile (ic->prev->op == '!' ? IC_LEFT (ic->prev) : IC_RIGHT (ic->prev), FALSE))
-              {
-                if (ic->prev->op =='!') /* Invert jump logic */
-                  {
-                    symbol *tmp = IC_TRUE (ic);
-                    IC_TRUE (ic) = IC_FALSE (ic);
-                    IC_FALSE (ic) = tmp;
-                  }
-                bitVectUnSetBit (OP_USES (IC_COND (ic)), ic->key);
-                ReplaceOpWithCheaperOp(&IC_COND (ic), ic->prev->op == '!' ? IC_LEFT (ic->prev) : IC_RIGHT (ic->prev));
-                (*change)++;
-              }
+          /* Don't do this for "if (--c)", it inhibits DJNZ generation */
+          if (!ic->prev->prev || ic->prev->prev->op != '-' || !IS_OP_LITERAL(IC_RIGHT(ic->prev->prev)))
+            {
+              sym_link *type = operandType (IC_RESULT (ic->prev));
+              if (ic->prev->op != CAST || IS_BOOL (type) || bitsForType (operandType (IC_RIGHT (ic->prev))) < bitsForType (type))
+                {
+                  if (!isOperandVolatile (ic->prev->op == '!' ? IC_LEFT (ic->prev) : IC_RIGHT (ic->prev), FALSE))
+                    {
+                      if (ic->prev->op == '!') /* Invert jump logic */
+                        {
+                          symbol *tmp = IC_TRUE (ic);
+                          IC_TRUE (ic) = IC_FALSE (ic);
+                          IC_FALSE (ic) = tmp;
+                        }
+                      bitVectUnSetBit (OP_USES (IC_COND (ic)), ic->key);
+                      ReplaceOpWithCheaperOp(&IC_COND (ic), ic->prev->op == '!' ? IC_LEFT (ic->prev) : IC_RIGHT (ic->prev));
+                      (*change)++;
+                    }
 /* There's an optimization opportunity here, but OP_USES doesn't seem to be */
 /* initialized properly at this point. - EEP 2016-08-04 */
 #if 0
-            else if (bitVectnBitsOn (OP_USES(IC_COND (ic))) == 1)
-              {
-                /* We can replace the iTemp with the original volatile symbol */
-                /* but we must make sure the volatile symbol is still accessed */
-                /* only once. */
-                bitVectUnSetBit (OP_USES (IC_COND (ic)), ic->key);
-                ReplaceOpWithCheaperOp(&IC_COND (ic), IC_RIGHT (ic->prev));
-                (*change)++;
-                /* Make previous assignment an assignment to self. */
-                /* killDeadCode() will eliminiate it. */
-                IC_RIGHT (ic->prev) = IC_RESULT (ic->prev);
-                IC_LEFT (ic->prev) = NULL;
-                ic->prev->op = '=';
-              }
+                  else if (bitVectnBitsOn (OP_USES(IC_COND (ic))) == 1)
+                    {
+                      /* We can replace the iTemp with the original volatile symbol */
+                      /* but we must make sure the volatile symbol is still accessed */
+                      /* only once. */
+                      bitVectUnSetBit (OP_USES (IC_COND (ic)), ic->key);
+                      ReplaceOpWithCheaperOp(&IC_COND (ic), IC_RIGHT (ic->prev));
+                      (*change)++;
+                      /* Make previous assignment an assignment to self. */
+                      /* killDeadCode() will eliminiate it. */
+                      IC_RIGHT (ic->prev) = IC_RESULT (ic->prev);
+                      IC_LEFT (ic->prev) = NULL;
+                      ic->prev->op = '=';
+                    }
 #endif
-          }
+                }
+            }
         }
     }
 
@@ -1933,7 +1939,6 @@ boolCast (iCode * ic, set * cseSet)
     case NE_OP:
     case AND_OP:
     case OR_OP:
-    case GETHBIT:
     case GETABIT:
       break;
     case BITWISEAND:
@@ -2099,7 +2104,6 @@ static int isSignedOp (iCode *ic)
     case NE_OP:
     case RRC:
     case RLC:
-    case GETHBIT:
     case GETABIT:
     case GETBYTE:
     case GETWORD:
