@@ -26,6 +26,7 @@
 #include "dbuf_string.h"
 
 #include "ralloc.h"
+#include "gen.h"
 #include "peep.h"
 
 extern DEBUGFILE dwarf2DebugFile;
@@ -98,21 +99,20 @@ pdk_genInitStartup (FILE *of)
   if (options.stack_loc >= 0)
     {
       fprintf (of, "\tmov\ta, #0x%02x\n", options.stack_loc);
-      fprintf (of, "\tmov\tsp, a\n");
+      fprintf (of, "\tmov.io\tsp, a\n");
     }
   else
     {
       fprintf (of, "\tmov\ta, #s_OSEG\n");
       fprintf (of, "\tadd\ta, #l_OSEG + 1\n");
       fprintf (of, "\tand\ta, #0xfe\n");
-      fprintf (of, "\tmov\tsp, a\n");
+      fprintf (of, "\tmov.io\tsp, a\n");
     }
 
   fprintf (of, "\tcall\t__sdcc_external_startup\n");
-  fprintf (of, "\tgoto\t__sdcc_gs_init_startup\n");
+  fprintf (of, "\tgoto\ts_GSINIT\n");
 
   tfprintf (of, "\t!area\n", STATIC_NAME);
-  fprintf (of, "__sdcc_gs_init_startup:\n");
 
   /* Init static & global variables */
   fprintf (of, "__sdcc_init_data:\n");
@@ -136,6 +136,8 @@ static void
 pdk_init (void)
 {
   asm_addTree (&asm_asxxxx_smallpdk_mapping);
+
+  pdk_init_asmops();
 }
 
 static void
@@ -200,7 +202,7 @@ _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
 static bool
 hasExtBitOp (int op, int size)
 {
-  return (false);
+  return (op == GETBYTE || op == SWAP && size == 1);
 }
 
 static const char *
@@ -268,6 +270,8 @@ PORT pdk13_port =
     0,
     pdknotUsedFrom,
     0,
+    0,
+    0,
   },
   /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
   {
@@ -314,6 +318,7 @@ PORT pdk13_port =
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -324,7 +329,7 @@ PORT pdk13_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
@@ -480,6 +485,7 @@ PORT pdk14_port =
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -490,7 +496,7 @@ PORT pdk14_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
@@ -646,6 +652,7 @@ PORT pdk15_port =
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -656,7 +663,7 @@ PORT pdk15_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
