@@ -10,10 +10,7 @@
 unsigned {type} game_menu_sel;
 unsigned {type} s_lin1;
 
-void game_menu_back(unsigned {type} f_start)
-#if defined(__SDCC_z80) || defined(__SDCC_z180) || defined(__SDCC_tlcs90) || defined(__SDCC_r2k) || defined(__SDCC_r3ka)
-__z88dk_fastcall
-#endif
+void game_menu_back(unsigned {type} f_start) __z88dk_fastcall
 {
     game_menu_sel = 0;
     s_lin1 = f_start;
