@@ -4,7 +4,7 @@
 
 #include <testfwk.h>
 
-#if defined(__SDCC_z80) || defined(__SDCC_z180) || defined(__SDCC_r2k) || defined(__SDCC_r3ka) || defined(__SDCC_tlcs90) || defined(__SDCC_ez80_z80)
+#if defined(__SDCC_z80) || defined(__SDCC_z180) || defined(__SDCC_r2k) || defined(__SDCC_r2ka) || defined(__SDCC_r3ka) || defined(__SDCC_tlcs90) || defined(__SDCC_ez80_z80)
 void write_bc(void) __preserves_regs(a, d, e, h, l, iyl, iyh) __naked
 {
 __asm
@@ -75,7 +75,7 @@ __endasm;
 
 void testRegs(void)
 {
-#if defined(__SDCC_z80) || defined(__SDCC_z180) || defined(__SDCC_r2k)|| defined(__SDCC_r3ka)
+#if defined(__SDCC_z80) || defined(__SDCC_z180) || defined(__SDCC_r2k) || defined(__SDCC_r2ka) || defined(__SDCC_r3ka) || defined(__SDCC_tlcs90) || defined(__SDCC_ez80_z80)
 	ASSERT(add_bc(0x55, 0xaa) == ({type})(0x55 + 0xaa));
 	ASSERT(add_de(0x55, 0xaa) == ({type})(0x55 + 0xaa));
 	ASSERT(add_hl(0x55, 0xaa) == ({type})(0x55 + 0xaa));
