@@ -47,6 +47,11 @@ void f1(void)
 
 #endif
 
+// Get FreeBSD version to skip part of test for known broken setjmp (FreeBSD bug #255320).
+#ifdef __FreeBSD__
+#include <sys/param.h>
+#endif
+
 void
 testJmp (void)
 {
@@ -70,9 +75,11 @@ testJmp (void)
       ASSERT (0);
     }
   ASSERT (exception == 1);
-
+  
+#if !defined(__FreeBSD__) || __FreeBSD_version > 1300139
   f1();
 #endif
+#endif
 
 // C99 might require setjmp to be a macro. The standard seems self-contradicting on this issue.
 //#ifndef setjmp
